function [retRes,retAlgo] = training(retAlgo,dat,lossType)


%  Does cross validation and returns the results as a data object
%  plus the updated group (after training models).

 disp(['training ' get_name(retAlgo) '.... '])
    
 if nargin==1 
     dat=data; % <-- in case data is not specified 
 end;          
 
 if nargin<3 
     lossType=[];  % <-- in case loss is not specified
 end 
  
 if isa(retAlgo.child{1},'cell') 
     retAlgo.child{1}=group(retAlgo.child{1}); % <-- convert retAlgo.child{1} to group
 end;
    
 [num,vecDim,outDim]=get_dim(dat); orig_name=get_name(dat); retRes=[];
  
 for loop=1:retAlgo.repeats % <-- do cross validation vecDim times

  rand('seed',loop);                
  if retAlgo.balanced     % <-- balance the number of positives/negatives labels in folds
   labels=get_y(dat); 
   fin1=find(labels==1); 
   fin2=find(labels==-1);
   
   pe1=randperm(sum(labels==1)); 
   pe1=fin1(pe1);
   
   pe2=randperm(sum(labels==-1)); 
   pe2=fin2(pe2);
   perm=[pe1 ;pe2];   
  else
   perm=randperm(num);
  end 

  for i=1:retAlgo.folds
    if retAlgo.train_on_fold==0
      tst=perm(i:retAlgo.folds:num);
      trn=setdiff(1:num,tst);
    else  % <--- take fraction as training data
      trn=perm(i:retAlgo.folds:num);
      tst=setdiff(1:num,trn);
    end
        
    if retAlgo.store_all 
        indx=i+(loop-1)*(retAlgo.folds); 
    else 
        indx=1; 
    end;
    
    [res retAlgo.child{indx}]=train(retAlgo.child{1},get(dat,trn));  
    res=dat;
    
    str=[' fold=' num2str(i+(loop-1)*(retAlgo.folds))]; 

    if retAlgo.output_train_error==0  % <--- test on left out fold
      dat=set_name(dat,[ orig_name ' -> cv' str]);      
      [res]=test(retAlgo.child{indx},get(dat,tst)); 
    else
      dat=set_name(dat,[ orig_name ' -> cv' str ' output_train_error=1']);   
    end
            
    if ~isempty(lossType)        % <--- calculate loss on results
      res=loss(res,lossType,[],1); 
    end
    res=make_cell(res);
    if ~isempty(retRes) 
        retRes={retRes{1:length(retRes)} res{1:length(res)}}; 
    else retRes=res; 
    end
    
  end;

  
end

if iscell(retRes) 
    retRes=group(retRes); 
end;

