function d =  testing(a,d)
  
%Extract computes the features of the test points with respect to
%the eigenvectors of the covariance matrix of training points x
%under the given kernel, with the appropriate centering in 
%feature space. The offset vector is calculated by the "train"
%function, and is used to make the calculations a bit faster.
%
% for additional information see Advances in Kernel Methods
%    by Scholkopf et. al., equations 20.30 - 20.34.
%
% Author:      Ben O'Loghlin
% Created:     18/09/00
% Updated:     18/09/00
%
% This code is released under the GNU Public License
% Copyright by The Australian National University

% set size of covariance matrix based on number of training points
% and number of test points

  
  if isempty(a.child)
    K=get_x(d)';
  else
    K=get_kernel(a.child,d,a.dat);   %% calc kernel
  end
  

%train_size = size(a.e_vec,1);
train_size = size(K,1);
test_size = size(K,2);
numfeatures = a.feat;
if (numfeatures==0)
  numfeatures=train_size;
end

% centering in feature space
% the rest of this program basically just computes 
% Eq 20.34 using some matrix factorization tricks to make it a 
% bit faster.

unit=ones(train_size)/train_size;
K_offset=zeros(test_size,train_size);

for (i=1:test_size)
  K_offset(i,:)=a.offset';
end

% alpha_norm is the row vector whose entries are the weight 
% of each eigenvector

alpha_norm = zeros(1,numfeatures);
for (i=1:train_size)
  for (j=1:numfeatures)
    alpha_norm(j)= alpha_norm(j) + a.e_vec(i,j);
  end
end

alpha_norm_matrix = zeros(train_size,numfeatures);
for (i=1:train_size)
  alpha_norm_matrix(i,:) = alpha_norm;
end

test_features = K'*(a.e_vec(:,1:numfeatures)-alpha_norm_matrix);

for (j=1:numfeatures)
  alpha_offset = a.offset'*a.e_vec(:,j);
  for (i=1:test_size)
    test_features(i,j) = test_features(i,j) + alpha_offset;
  end
end
  
d=set_x(d,test_features); d.name=[get_name(d) ' -> ' get_name(a)]; 

 

 

