function [results,a] =  training(a,d)
       
  % [results,algorithm] =  training(algorithm,data,loss)

  disp(['training ' get_name(a) '.... '])
  
  %
  %Train computes the eigensystem of K_n, the normalized 
  %gram matrix induced by the kernel on the training set. 
  %Note that the eigenvectors are not normalized, for reasons of 
  %numerical stability in the case where some eigenvalues are close
  %to zero.
  %This function also returns an M*1 matrix called 'offset' which
  %represents the offset of the eigenvectors in K-space due to
  %centering of the training data. In the terminology of eq 20.34 in 
  %Advances in Kernel Methods, Scholkopf et al,
  %   offset = RK - RK1m, where R is the row vector ones(1,M) and 
  %                       1m is the matrix ones(M)/M.
  %
  % Author:      Ben O'Loghlin
  % Created:     10/09/00
  % Updated:     18/09/00
  %
  % This code is released under the GNU Public License
  % Copyright by The Australian National University
  
  % generate normalized K matrix
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

  % create the covariance matrix
  % set size of covariance matrix based on number of input points

  if isempty(a.child)
    K=get_x(d);
  else
    K=get_kernel(a.child,d,[]);   %% calc kernel
  end
  cov_size=length(K);
  
  % centering in feature space, basically implements Scholkopf
  % et. al.'s Eq 20.32 but using some matrix factorizations to 
  % make it a bit faster.

  if 1
    unit_vec = ones(cov_size,1);
    K_= K*unit_vec/cov_size;
    K_sum = (unit_vec'*K_)/cov_size;
    
    K_n = ((K - unit_vec*K_') - K_*unit_vec') + ones(cov_size)*K_sum;
    a.offset = K_ - K_sum*unit_vec*cov_size;
  else %% CRAP I TRIED BELOW
    a.offset=ones(cov_size,1);
    K_n=K*cov_size; 
  end
    
  % don't know how many feature? compute rank first 
  if (a.feat == 0)
    a.feat=rank(K_n/cov_size);  % (i know it's slow...)
  end

  % compute eigensystem
  if (a.feat == 0)
    [a.e_vec, a.e_val]= eig(K_n/cov_size);
    a.e_val = real(diag(a.e_val));
  else
    %[a.e_vec, a.e_val]= eigs(K_n/cov_size,a.feat);
    
    opts.disp=0;
    [a.e_vec, a.e_val]= eigs(K_n/cov_size,a.feat,'LM',opts);
    
    a.e_val = real(diag(a.e_val));
  end
  
  
  %normalize the eigenvector by the square root of the eigenvalues
  for i=1:size(a.e_vec,2)
    a.e_vec(:,i) = a.e_vec(:,i)/sqrt(a.e_val(i));
  end 
    
 a.dat=d;
  results=test(a,d);
  







