
function a = odata(p1,p2) 
  
% ODATA data object for streams
%  stores data in a history of size p1, into two components X and Y which generally mean
%  input (for X) and output (for Y). To create an object use
%     
%  odata;    creates empty object with default history = 50;
%  or 
%  odata({'xin=xfilename','yin=yfilename'}); or data('xfilename') when there is no output labels .
%  or 
%  odata('name',{'xin=xfilename','yin=yfilename'}); 
%  odata(10,{'xin=xfilename','yin=yfilename'});  % history = 10
%
% Attributes:
%	history=50;
%	xin = [];
%	yin = [];
%	X=[]; 
%	Y=[];
%
% Methods:
%  d=get(d,ind,find):      return data,index only (ind)examples,(find)features 
%  x=get_x(d,ind,find)     return x,index only (ind)examples,(find)features 
%  y=get_y(d,ind,find)     return y,index only (ind)examples,(find)features 
%  d=set_x(d,x,ind,find)   set inputs indexed by (ind)examples,(find)features
%  d=set_y(d,y,ind,find)   set outputs indxed by (ind)examples,(find)features
%  [ind,find]=get_index(d) returns example and feature indices
%  [l,n,k]=get_dim(d):     returns number of examples(l),features(n),outputs(k)
%
% Example: get_x(data([1:5;6:10;11:15]),[1 3],[3:5])

% ============================================= 
% =============== DEFAULTS ====================	
% ============================================= 

	a.history=50;
	a.xin = [];
	a.yin = [];
	a.xfid=[];
    a.yfid=[];
    a.consumed =0;
	a.X=[]; 
	a.Y=[];
    name='odata (empty)';
      
	if nargin ==1
		if(isa(p1,'char'))
		       a.xin=p1;
		else
		 if(isa(p1,'cell'))
	    	% hyper evaluation
		 		hyper=p1;
        		eval_hyper;
         else
          		a.history=p1;
         end
        end
           
    else
    	if nargin ==2
    		if(isa(p1,'char'))
		       name=p1;
			else
	    		a.history=p1;
			end
	    	% hyper evaluation
	    	hyper=p2;
   			eval_hyper;
		end    	
    end
       
  
  p=algorithm(name); 
  % ----------- open -------------
  % i did not decided 
  % ??? I don't know about this
  % ------------------------------
  p.is_data=0;
  % ------------------------------
  
  a= class(a,'odata',p);
  