
function a = svmtrack(hyper) 
% SVM TRACKING object
% 
% A=SVMTRACK(H) returns an svmt object initialized with hyperparameters H. 
% stores all intermediate outputs in member track.
%
% Hyperparameters, and their defaults
%  child=kernel         -- the kernel is stored as a member called "child"
%  C=Inf                -- the soft margin C parameter
%  ridge=1e-13          -- a ridge on the kernel
%  balanced_ridge=0     -- for unbalanced data
%  nu = 0               -- bernhard's nu svm parameter
%  optimizer='default'  -- other choices={andre,quadprog,svmlight,libsvm}
%  alpha_cutoff=-1;     -- keep alphas with abs(a_i)>max(a)/alpha_cutoff
%                          default keeps all alphas, another
%                          reasonable choice is e.g alpha_cutoff=1e5 to remove
%                          zero alphas (i.e non-SVs) to speed up computations.
% 
% Model
%  alpha                -- the weights
%  b0                   -- the threshold
%  Xsv                  -- the Support Vectors
%
% Methods:
%  train, test, get_w , get_track()

  %hyperparams 
  a.child=kernel;
  a.C=Inf;
  
  a.track=[];
  
  
  a.ridge=1e-13;  
  a.balanced_ridge=0;
  a.nu = 0;
  a.optimizer='default';
  a.alpha_cutoff=-1;
  
  
  % model 
  a.alpha=[];
  a.b0=0;
  a.Xsv=[];
  
  p=algorithm('svmtrack');
  a= class(a,'svmtrack',p);

  a.algorithm.alias={'kern','child'}; % other way of talking about kernel
  
 if nargin==1,
    eval_hyper;
 end;

 





