
function [d,a] =  training(a,d)
  
  if a.algorithm.verbosity>0
    disp(['training ' get_name(a) '.... '])
  end
  
  opt=a.optimizer;
  if strcmp(a.optimizer,'default')
    l=length(d.Y);
    if l<=200  %% more than 200 examples, then use svmlight
     opt='andre';
    else
     opt='svmlight';
    end
  end
  if a.nu~=0,
      opt='libsvm';
  end;
 
 switch opt
  
  case {'andre'}   % ****************************************---
   
   [K,a.child]=train(a.child,d); K=K.X;    %% calc kernel
   K=add_ridge(K,a,d); Y=get_y(d); K=K.*(Y*Y');
   [alpha,b] = quadsolve(K,-ones(size(K,1),1),Y',0,a.C); b0 = -b;
   alpha= alpha .* Y;
   
  case {'quadprog'}  % ****************************************---
  
   [K,a.child]=train(a.child,d); K=K.X;    %% calc kernel
   K=add_ridge(K,a,d);  l=size(K,1);  Y=get_y(d); K=K.*(Y*Y');
   opts= optimset('display','off','MaxIter',10000,'LargeScale','off'); 
   [alpha,fval,exit,out,lambda] = quadprog(K,-ones(l,1),[],[],Y',0, zeros(l,1),a.C*ones(l,1),[],opts);
   b0=lambda.eqlin(1);  alpha= alpha .* Y;
  
  case {'svmlight'}  % ****************************************---
     
   a.child.dat=d; %% kernel must store data now..
   [X Y]=get_xy(d);
   if strcmp(a.child.ker,'linear') ker=1; p1=1; end;
   if strcmp(a.child.ker,'poly') ker=2; p1=a.child.kerparam; end;
   if strcmp(a.child.ker,'rbf') 
     ker=3; p1=a.child.kerparam; p1=(2*p1^2);   
   end;
   if strcmp(a.child.ker,'weighted_linear') 
     ker=1; p1=1; tmp = a.child.kerparam; 
     X=X .* repmat(tmp,size(X,1),1); % weight data according to params
   end;
   if strcmp(a.child.ker,'custom') | strcmp(a.child.ker,'from_data')
     X=get_kernel(a.child,d,d);  %% calc kernel
     X = [[1:size(X,1)]' X]; ker=4; p1=1;
   end;
   X=full(X);
   [as b0 ind] = svm_light(X,Y,a.C,a.ridge,a.balanced_ridge,ker,p1, ...
			   max(0,a.algorithm.verbosity-1));
   if a.algorithm.verbosity>1 disp('done!'); end;
   alpha=zeros(size(X,1),1); alpha(ind+1)=as;
 
   
  case {'libsvm'} % ****************************************---
  
   
   a.child.dat=d; %% kernel must store data now..
    if a.nu==0,
        svm_type = 0; C = min([a.C,10000]); nu=0;
    else
        svm_type = 1; nu = a.nu; a.C = -1; C=1;
    end;
   deg = 0; coef0 = 0;
   
   %% default values for libsvm
   
   cache_size = 40; eps = 0.001; shrinking=1;
   nr_weight = 0; weight_label =0; weight = 1;
   epsilon = 1; gamma=1;
   
   if strcmp(a.child.ker,'linear')
       kernel_type = 0;
   end;
   if strcmp(a.child.ker,'poly')
       kernel_type = 1; deg = a.child.kerparam; coef0 = 1;
   end;
   if strcmp(a.child.ker,'rbf'),
         kernel_type = 2; sigma = a.child.kerparam; gamma = 1/(2*sigma^2);
   end;
   if a.balanced_ridge~=0,
     disp('Warning: balanced ridge only for C-SVC (libsvm).');
     C1 = (a.balanced_ridge)*length(Y)/length(find(Y==1));
     Cm1 = (1-a.balanced_ridge)*length(Y)/length(find(Y==-1));
   end;
   Y=get_y(d); X=get_x(d);
    [alpha,b0,Xsv,eps,CC] = svmlibtrain(X,Y,svm_type,kernel_type,deg,gamma,coef0,nu,cache_size,C,eps,epsilon,shrinking,nr_weight,weight_label,weight,0);    
       b0=-b0;% from libsvm
   % Reorder the alpha to have the same Xsv for all runs
   % this is important for one_vs_rest
   alphatmp = zeros(size(Xsv,1),1);
   indtmp = find(Xsv(:,size(Xsv,2))~=0);
   indtmp2 = Xsv(indtmp,size(Xsv,2));
   alphatmp(indtmp2) = alpha(indtmp)*Y(1);
   alpha = alphatmp;   
    % Mistake? in libsvm, we have to scale alpha and b by Y(1)
   b0 = b0*Y(1);    
   a.C = CC; 
 end
 
 a.b0=b0;
 f=find(abs(alpha)>a.alpha_cutoff);
 a.alpha=alpha(f);
 a.Xsv=get(d,f);
      
 if a.algorithm.do_not_evaluate_training_error
   d=set_x(d,get_y(d)); 
 else
   d=test(a,d);
 end
  
 
